@@ -7,22 +7,34 @@ require Exporter;
 require DynaLoader;
 
 use vars qw(@ISA %EXPORT_TAGS @EXPORT_OK $VERSION);
-@ISA = qw(Exporter DynaLoader);
+@ISA = qw(DynaLoader);
 
-$VERSION = '0.32_02';
+$VERSION = '0.32_03';
 
 %EXPORT_TAGS = (
     'all' => [qw(
         &clear &compare &copy &generate &generate_random &generate_time
-        &is_null &parse &time &type &unparse &unparse_lower
-        &unparse_upper &uuid &uuid1 &uuid4 &variant
+        &generate_v1 &generate_v4 &is_null &parse &time &type &unparse
+        &unparse_lower &unparse_upper &uuid &uuid1 &uuid4 &variant
     )],
 );
 
-@EXPORT_OK = ( @{$EXPORT_TAGS{'all'}} );
+@EXPORT_OK = ( @{$EXPORT_TAGS{'all'}}, '&randnext' );
 
 bootstrap UUID $VERSION;
 
+sub import {
+    # :persist=FOO
+    for (my $i=scalar(@_)-1 ; $i>0 ; --$i) {
+        my $v = $_[$i];
+        if (length($v) > 9 and substr($v,0,9) eq ':persist=') {
+            _persist(substr($v, 9));
+            splice @_, $i, 1;
+        }
+    }
+    goto &Exporter::import;
+}
+
 # Preloaded methods go here.
 
 1;
@@ -40,9 +52,12 @@ UUID - Universally Unique Identifier library for Perl
     $string = uuid1();  # generate UUID string, always v1
     $string = uuid4();  # generate UUID string, always v4
 
-    UUID::generate($uuid);               # new binary UUID; prefer random
-    UUID::generate_random($uuid);        # new binary UUID; use random
-    UUID::generate_time($uuid);          # new binary UUID; use time
+    UUID::generate_v1($uuid);            # new version 1 binary UUID
+    UUID::generate_v4($uuid);            # new version 4 binary UUID
+
+    UUID::generate($uuid);               # new binary UUID; prefer v4
+    UUID::generate_time($uuid);          # alias for generate_v1()
+    UUID::generate_random($uuid);        # alias for generate_v4()
 
     UUID::unparse($uuid, $string);       # stringify $uuid; system casing
     UUID::unparse_lower($uuid, $string); # force lowercase stringify
@@ -72,10 +87,10 @@ communication between the servers, and without fear of a name clash.
 The generated UUIDs can be reasonably expected to be unique within a
 system, and unique across all systems, and are compatible with those
 created by the Open Software Foundation (OSF) Distributed Computing
-Environment (DCE) utility uuidgen.
+Environment (DCE).
 
-All generated UUIDs are either type 1 from B<UUID::generate_time()>, or
-type 4 from B<UUID::generate_random()>. And all are variant 1, meaning
+All generated UUIDs are either version 1 from B<generate_v1()>, or
+version 4 from B<generate_v4()>. And all are variant 1, meaning
 compliant with the OSF DCE standard as described in RFC4122.
 
 =head1 FUNCTIONS
@@ -148,9 +163,10 @@ I</dev/random> in non-blocking mode.
 Generates a new version 1 binary UUID which uses the current time and the
 local ethernet MAC address, if available.
 
-If the MAC address is not available, it is replaced randomly from best
-source with the multicast bit set to avoid conflict with addresses
-returned from network cards.
+If the MAC address is not available at startup, it is generated randomly
+and used through the lifetime of the process. The multicast bit of this
+address is set to avoid conflict with addresses returned from network
+cards.
 
 This algorithm used to be the default used to generate UUIDs, but
 because of inclusion of the ethernet MAC address, it can leak
@@ -160,6 +176,14 @@ This can cause privacy problems in some applications, so the
 B<generate()> function only uses this algorithm if a high-quality source
 of randomness is not available.
 
+=head2 B<generate_v1(> I<$uuid> B<)>
+
+Alias for B<generate_time()>.
+
+=head2 B<generate_v4(> I<$uuid> B<)>
+
+Alias for B<generate_random()>.
+
 =head2 B<is_null(> I<$uuid> B<)>
 
 Compares the value of I<$uuid> to the NULL UUID.
@@ -241,21 +265,60 @@ Known variants:
     2  Microsoft
     3  Other
 
-=head1 UUID LIBRARY
+=head1 MAINTAINING STATE
 
-Prior to version 0.32, UUID required libuuid or similar be installed
-first. This is no longer the case. UUID now builds against a bundled
-copy of the e2fsprogs UUID code.
+Internal state is optionally maintained for version 1 UUIDs via a file
+designated by the B<:persist> export tag (see EXPORTS), if the path
+exists and the user running the process has read/write access.
+
+If the file doesn't exist, it will be created if the directory path
+exists.
+
+The file records various internal states at the time the last UUID is
+generated, preventing future instances from overlapping the prior UUID
+sequence. This allows the sequence to absolutely survive reboots and,
+more importantly, backwards resetting of system time.
+
+If B<:persist> is not used, time resets will still be detected while the
+module is loaded, and handled accordingly. And since startup in this
+case is randomized anyway, the chance of overlap is low but still exists
+since the randomized clock_seq field is only 14 bits wide.
+
+Note that use of this feature incurs a serious performance penalty,
+upwards of 90% on tested platforms.
+
+Note too that use of a random MAC greatly reduces the chance of overlap,
+making the randomized parts 62 bits wide at start.
 
 =head1 EXPORTS
 
 None by default. All functions may be imported in the usual manner,
-either individually or all at once using the "I<:all>" tag.
+either individually or all at once using the B<:all> tag.
+
+=head2 B<:persist>=F<path/to/state.txt>
+
+Path to version 1 state maintenance file. (See B<MAINTAINING STATE>.)
+
+If the file does not exist, it will be created provided the path
+exists.
+
+If the file cannot be opened, cannot be created, or is a symlink, UUID
+will ignore it. No state will be maintained.
+
+B<WARNING>: Do not point B<:persist> into a public directory. See
+CVE-2013-4184. UUID attempts to avoid this, but nothing is foolproof.
+Only YOU can prevent symlink attacks!
 
 =head1 THREAD SAFETY
 
 This module is believed to be thread safe.
 
+=head1 UUID LIBRARY
+
+Releases prior to UUID-0.32 required libuuid or similar be installed
+first. This is no longer the case. UUID now builds against a bundled
+copy of the e2fsprogs UUID code.
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is Copyright (c) 2014-2024 by Rick Myers.
