@@ -3,11 +3,13 @@
 #
 # check a bunch of stuff.
 # write #define/#undef to file given in ARGV[0].
+# now also writes #include's too.
 #
 
 use strict;
 use warnings;
 use Config;
+use File::Temp ();
 use Devel::CheckLib;
 
 my ($out, $flagfile) = @ARGV;
@@ -37,21 +39,33 @@ sub generate_conf {
 
     for my $check ( 1, 0 ) {
         try_headers($check, \$longest, $defines, qw(
+            HAVE_CTYPE_H         ctype.h
+            HAVE_ERR_H           err.h
+            HAVE_ERRNO_H         errno.h
+            HAVE_FCNTL_H         fcntl.h
             HAVE_INTTYPES_H      inttypes.h
+            HAVE_IO_H            io.h
             HAVE_NETINET_IN_H    netinet/in.h
             HAVE_NET_IF_DL_H     net/if_dl.h
             HAVE_NET_IF_H        net/if.h
+            HAVE_PROCESS_H       process.h
+            HAVE_STDINT_H        stdint.h
+            HAVE_STDIO_H         stdio.h
             HAVE_STDLIB_H        stdlib.h
+            HAVE_STRING_H        string.h
             HAVE_SYS_FILE_H      sys/file.h
             HAVE_SYS_IOCTL_H     sys/ioctl.h
             HAVE_SYS_RANDOM_H    sys/random.h
             HAVE_SYS_RESOURCE_H  sys/resource.h
             HAVE_SYS_SOCKET_H    sys/socket.h
             HAVE_SYS_SOCKIO_H    sys/sockio.h
+            HAVE_SYS_STAT_H      sys/stat.h
             HAVE_SYS_SYSCALL_H   sys/syscall.h
             HAVE_SYS_TIME_H      sys/time.h
+            HAVE_SYS_TYPES_H     sys/types.h
             HAVE_SYS_UN_H        sys/un.h
             HAVE_SYS_WAIT_H      sys/wait.h
+            HAVE_TIME_H          time.h
             HAVE_UNISTD_H        unistd.h
             HAVE_WINDOWS_H       Windows.h
         ));
@@ -61,7 +75,9 @@ sub generate_conf {
         try_getrandom      ($check, \$longest, $defines);
         try_getrlimit      ($check, \$longest, $defines);
         try_jrand48        ($check, \$longest, $defines);
+        try_lstat          ($check, \$longest, $defines);
         try_srandom        ($check, \$longest, $defines);
+        try_symlink        ($check, \$longest, $defines);
         try_sysconf        ($check, \$longest, $defines);
 
         try_sa_len         ($check, \$longest, $defines);
@@ -88,7 +104,7 @@ sub try_headers {
     my ($check, $long, $defs, %try) = @_;
     map {
         check_header($check, $long, $defs, $_, $try{$_})
-    } sort keys %try;
+    } sort {$try{$a} cmp $try{$b}} keys %try;
 }
 
 sub check_header {
@@ -106,6 +122,7 @@ sub check_header {
         debug  => 0,
     )) {
         push @$defs, "#define $def 1";
+        push @$defs, "#include <${hdr}>";
         print $msg, ' ..', '.'x$more, ' found';
     }
     else {
@@ -114,15 +131,36 @@ sub check_header {
     }
 }
 
+sub gen_confh {
+    my ($defs) = @_;
+    my ($fh, $fname) = File::Temp::tempfile(
+        'assertconfXXXXXXXX', SUFFIX => '.h', UNLINK => 0
+    );
+    print $fh $_ for @$defs;
+    close $fh;
+    return $fname;
+}
+
+sub gen_confh_fh {
+    my ($defs) = @_;
+    my ($fh, $fname) = File::Temp::tempfile(
+        'assertconfXXXXXXXX', SUFFIX => '.h', UNLINK => 0
+    );
+    print $fh $_ for @$defs;
+    return $fname, $fh;
+}
+
 sub try_getdtablesize {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking getdtablesize()';
+    my $msg = 'Checking getdtablesize()';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
+    my $conf = gen_confh($defs);
     if ( check_lib(
-        header   => [qw(unistd.h)],
+        header   => $conf,
         lib      => 'c', # always do at least this
+        ccflags  => '-I.',
         debug    => 0,
         function => q{
             int r;
@@ -138,17 +176,20 @@ sub try_getdtablesize {
         push @$defs, '#undef HAVE_GETDTABLESIZE';
         print $msg, ' ..', '.'x$more, ' missing';
     }
+    unlink $conf;
 }
 
 sub try_getentropy {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking getentropy()';
+    my $msg = 'Checking getentropy()';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
+    my $conf = gen_confh($defs);
     if ( check_lib(
-        header   => [qw(unistd.h)],
+        header   => $conf,
         lib      => 'c', # always do at least this
+        ccflags  => '-I.',
         debug    => 0,
         function => q{
             unsigned char p[4];
@@ -165,17 +206,20 @@ sub try_getentropy {
         push @$defs, '#undef HAVE_GETENTROPY';
         print $msg, ' ..', '.'x$more, ' missing';
     }
+    unlink $conf;
 }
 
 sub try_getrandom {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking getrandom()';
+    my $msg = 'Checking getrandom()';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
+    my $conf = gen_confh($defs);
     if ( check_lib(
-        header   => [qw(sys/random.h)],
+        header   => $conf,
         lib      => 'c', # always do at least this
+        ccflags  => '-I.',
         debug    => 0,
         function => '
             unsigned char p[4];
@@ -192,17 +236,20 @@ sub try_getrandom {
         push @$defs, '#undef HAVE_GETRANDOM';
         print $msg, ' ..', '.'x$more, ' missing';
     }
+    unlink $conf;
 }
 
 sub try_getrlimit {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking getrlimit()';
+    my $msg = 'Checking getrlimit()';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
+    my $conf = gen_confh($defs);
     if ( check_lib(
-        header   => [qw(sys/resource.h)],
+        header   => $conf,
         lib      => 'c', # always do at least this
+        ccflags  => '-I.',
         debug    => 0,
         function => '
             struct rlimit rl;
@@ -219,17 +266,20 @@ sub try_getrlimit {
         push @$defs, '#undef HAVE_GETRLIMIT';
         print $msg, ' ..', '.'x$more, ' missing';
     }
+    unlink $conf;
 }
 
 sub try_jrand48 {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking jrand48()';
+    my $msg = 'Checking jrand48()';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
+    my $conf = gen_confh($defs);
     if ( check_lib(
-        header   => [qw(stdlib.h)],
+        header   => $conf,
         lib      => 'c', # always do at least this
+        ccflags  => '-I.',
         debug    => 0,
         function => q{
             unsigned short seed[3];
@@ -246,17 +296,56 @@ sub try_jrand48 {
         push @$defs, '#undef HAVE_JRAND48';
         print $msg, ' ..', '.'x$more, ' missing';
     }
+    unlink $conf;
+}
+
+sub try_lstat {
+    my ($check, $long, $defs) = @_;
+    my $msg = 'Checking lstat()';
+    my $len = length $msg;
+    if ($check) { $$long = $len if $len > $$long; return; }
+    my $more = $$long - $len;
+    my $conf = gen_confh($defs);
+
+    my $Werror = $WIN32 ? '-WX'  : '-Werror';
+    my $lib    = $WIN32 ? 'ucrt' : 'c';
+
+    if ( check_lib(
+        header   => $conf,
+        lib      => $lib, # always do at least this
+        ccflags  => "-I. $Werror",
+        debug    => 0,
+        function => q{
+            struct stat buf;
+            int    rc, islink;
+            rc     = lstat("foo", &buf);
+            islink = ((buf.st_mode & S_IFMT) == S_IFLNK);
+            return 0;
+            (void)rc;
+            (void)islink;
+        },
+    )) {
+        push @$defs, '#define HAVE_LSTAT 1';
+        print $msg, ' ..', '.'x$more, ' found';
+    }
+    else {
+        push @$defs, '#undef HAVE_LSTAT';
+        print $msg, ' ..', '.'x$more, ' missing';
+    }
+    unlink $conf;
 }
 
 sub try_srandom {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking srandom()';
+    my $msg = 'Checking srandom()';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
+    my $conf = gen_confh($defs);
     if ( check_lib(
-        header   => [qw(stdlib.h)],
+        header   => $conf,
         lib      => 'c', # always do at least this
+        ccflags  => '-I.',
         debug    => 0,
         function => q{
             unsigned int s = 8675309;
@@ -271,17 +360,62 @@ sub try_srandom {
         push @$defs, '#undef HAVE_SRANDOM';
         print $msg, ' ..', '.'x$more, ' missing';
     }
+    unlink $conf;
+}
+
+sub try_symlink {
+    my ($check, $long, $defs) = @_;
+    my $msg = 'Checking symlink()';
+    my $len = length $msg;
+    if ($check) { $$long = $len if $len > $$long; return; }
+    my $more = $$long - $len;
+    my $conf = gen_confh($defs);
+
+    my $has_symlink = eval {
+        symlink $conf, 'assertlibSYMLINK.h';
+        return 1 if -l 'assertlibSYMLINK.h';
+        return 0;
+    };
+
+    my $Werror = $WIN32 ? '-WX'  : '-Werror';
+    my $lib    = $WIN32 ? 'ucrt' : 'c';
+
+    if ( check_lib(
+        header   => $conf,
+        lib      => $lib, # always do at least this
+        ccflags  => "-I. $Werror",
+        debug    => 0,
+        function => q{
+            struct stat buf;
+            int    rc;
+            rc = lstat("assertlibSYMLINK.h", &buf);
+            if ((buf.st_mode & S_IFMT) == S_IFLNK)
+                return 0;
+            return 1;
+            (void)rc;
+        },
+    )) {
+        push @$defs, '#define HAVE_SYMLINK 1';
+        print $msg, ' ..', '.'x$more, ' found';
+    }
+    else {
+        push @$defs, '#undef HAVE_SYMLINK';
+        print $msg, ' ..', '.'x$more, ' missing';
+    }
+    unlink $conf, 'assertlibSYMLINK.h';
 }
 
 sub try_sysconf {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking sysconf()';
+    my $msg = 'Checking sysconf()';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
+    my $conf = gen_confh($defs);
     if ( check_lib(
-        header   => [qw(unistd.h)],
+        header   => $conf,
         lib      => 'c', # always do at least this
+        ccflags  => '-I.',
         debug    => 0,
         function => q{
             long r;
@@ -297,17 +431,20 @@ sub try_sysconf {
         push @$defs, '#undef HAVE_SYSCONF';
         print $msg, ' ..', '.'x$more, ' missing';
     }
+    unlink $conf;
 }
 
 sub try_sa_len {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking sa_len member';
+    my $msg = 'Checking sa_len member';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
+    my $conf = gen_confh($defs);
     if ( check_lib(
-        header   => [qw(net/if.h stdio.h err.h)],
+        header   => $conf,
         lib      => 'c', # always do at least this
+        ccflags  => '-I.',
         debug    => 0,
         function => q{
             struct ifreq ifr;
@@ -324,11 +461,12 @@ sub try_sa_len {
         push @$defs, '#undef HAVE_SA_LEN';
         print $msg, ' ..', '.'x$more, ' missing';
     }
+    unlink $conf;
 }
 
 sub try_win32_native {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking win32 native';
+    my $msg = 'Checking win32 native';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
@@ -353,35 +491,35 @@ sub try_win32_native {
 
 sub try_threads {
     my ($check, $long, $defs) = @_;
-    my $msg = join ' ', 'Checking thread storage';
+    my $msg = 'Checking thread storage';
     my $len = length $msg;
     if ($check) { $$long = $len if $len > $$long; return; }
     my $more = $$long - $len;
     # can we skip this if not -Dusethreads ?
 
-    open my $fh, '>', 'assertlibTHREADS.c' or die $!;
-    print $fh q{
+    my ($conf1, $fh1) = gen_confh_fh($defs);
+    print $fh1 q{
         int dec(void) {
           static __thread int foo = 3;
           return --foo;
         }
     };
-    close $fh;
+    close $fh1;
 
-    open $fh, '>', 'assertlibDECLSPEC.c' or die $!;
-    print $fh q{
+    my ($conf2, $fh2) = gen_confh_fh($defs);
+    print $fh2 q{
         int dec(void) {
           static __declspec(thread) int foo = 3;
           return --foo;
         }
     };
-    close $fh;
+    close $fh2;
 
     my $Werror = $WIN32 ? '-WX'  : '-Werror';
     my $lib    = $WIN32 ? 'ucrt' : 'c';
 
     if ( check_lib(
-        header   => [qw(assertlibTHREADS.c)],
+        header   => $conf1,
         lib      => $lib, # always do at least this
         ccflags  => "-I. $Werror",
         debug    => 0,
@@ -397,7 +535,7 @@ sub try_threads {
         print $msg, ' ..', '.'x$more, ' __thread';
     }
     elsif ( check_lib(
-        header   => [qw(assertlibDECLSPEC.c)],
+        header   => $conf2,
         lib      => $lib, # always do at least this
         ccflags  => "-I. $Werror",
         debug    => 0,
@@ -416,7 +554,7 @@ sub try_threads {
         push @$defs, '#undef TLS';
         print $msg, ' ..', '.'x$more, ' none';
     }
-    unlink qw(assertlibTHREADS.c assertlibDECLSPEC.c);
+    unlink $conf1, $conf2;
 }
 
 sub write_conf {
