@@ -2,10 +2,7 @@
 # Patch given file with DATA hunks and print to STDOUT.
 #
 #
-# This patches uuid_compare() to myuuid_compare().
-# For some reason, on FreeBSD, uuid_compare is linking
-# to something else, possibly the system libuuid,
-# even though we dont explicitly link it.
+# Patch to localize.
 #
 use strict;
 use warnings;
@@ -31,14 +28,23 @@ print $out;
 exit 0;
 
 __END__
---- usrc/uuid/compare.c	2023-11-29 23:03:41.791859643 -0500
-+++ usrcP/uuid/compare.c	2023-12-04 03:18:57.852413604 -0500
-@@ -40,7 +40,7 @@
+--- usrc/uuid/compare.c	Wed Nov 29 23:03:41 2023
++++ usrcP/uuid/compare.c	Wed Dec  6 00:54:07 2023
+@@ -40,12 +40,12 @@
  
  #define UUCMP(u1,u2) if (u1 != u2) return((u1 < u2) ? -1 : 1);
  
 -int uuid_compare(const uuid_t uu1, const uuid_t uu2)
-+int myuuid_compare(const uuid_t uu1, const uuid_t uu2)
++int myuuid_compare(const myuuid_t uu1, const myuuid_t uu2)
  {
- 	struct uuid	uuid1, uuid2;
+-	struct uuid	uuid1, uuid2;
++	struct myuuid	uuid1, uuid2;
  
+-	uuid_unpack(uu1, &uuid1);
+-	uuid_unpack(uu2, &uuid2);
++	myuuid_unpack(uu1, &uuid1);
++	myuuid_unpack(uu2, &uuid2);
+ 
+ 	UUCMP(uuid1.time_low, uuid2.time_low);
+ 	UUCMP(uuid1.time_mid, uuid2.time_mid);
+
