@@ -2,7 +2,7 @@
 # Patch given file with DATA hunks and print to STDOUT.
 #
 #
-# Patching types in pack.c.
+# Patch to localize.
 #
 use strict;
 use warnings;
@@ -28,15 +28,18 @@ print $out;
 exit 0;
 
 __END__
-diff -aurN usrcP/uuid/pack.c usrcP.targ/uuid/pack.c
---- usrcP/uuid/pack.c	2023-12-02 17:58:13.891969000 +0200
-+++ usrcP.targ/uuid/pack.c	2023-12-02 15:28:29.183074000 +0200
-@@ -38,7 +38,7 @@
+--- usrc/uuid/pack.c	Wed Nov 29 23:03:41 2023
++++ usrcP/uuid/pack.c	Wed Dec  6 00:30:46 2023
+@@ -36,9 +36,9 @@
+ #include <string.h>
+ #include "uuidP.h"
  
- void uuid_pack(const struct uuid *uu, uuid_t ptr)
+-void uuid_pack(const struct uuid *uu, uuid_t ptr)
++void myuuid_pack(const struct myuuid *uu, myuuid_t ptr)
  {
 -	uint32_t	tmp;
 +	myuint32_t	tmp;
  	unsigned char	*out = ptr;
  
  	tmp = uu->time_low;
+
