@@ -32,7 +32,7 @@ if ($file eq 'uuid/uuid.h') {
     $from = catfile( 'ulib', $file . '.new' );
     $to   = catfile( 'ulib', $file );
 }
-elsif ($file eq 'config.h') {
+elsif ($file eq 'uuconfig.h') {
     $from = catfile( 'ulib', $file . '.new' );
     $to   = catfile( 'ulib', $file );
 }
